PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

echo "SUPPORT SCRIPT 20050315-1"

echo ""
echo "HOSTNAME:"
hostname
echo ""
echo "CLISP PATH INFO:"
ls -l /usr/local/bin/clisp
ls -l /usr/bin/clisp
echo "WHICH:"
which clisp

echo "LISP PROGS:"
ls -l /home/grid/*.lisp
ls -l /usr/sbin/*.lisp

echo "INETD PROC:"
ps -ax | grep inetd
echo "SERVICES:"
grep sgp /etc/services
echo "INETD.CONF:"
grep sgp /etc/inetd.conf

echo "MY_IP:"
cat /home/grid/my_ip.dat; echo ""
echo "TRACK_IP:"
cat /home/grid/track_ip.dat; echo ""
echo "CPU_FREQ:"
cat /home/grid/cpu_freq.dat; echo ""
echo "TOKEN CHECKSUM:"
cksum /home/grid/token.dat
echo "TOP:"
cat /home/grid/top.dat

echo "ROUTING TABLE:"
netstat -rn
echo "INTERFACES:"
ifconfig -a
echo "XINETD.CONF:"
cat /etc/xinetd.conf
echo "HOSTS.ALLOW:"
grep lisp /etc/hosts.allow
echo "UNAME:"
uname -rs
echo "UID's:"
id grid
id nobody
echo "GRID HOME:"
ls -ld /home/grid
echo "DAT FILE PERM.:"
ls -l /home/grid/*.dat
